!     -*- f90 -*-
!     This file is autogenerated with f2py (version:2)
!     It contains Fortran 90 wrappers to fortran functions.

      
      subroutine f2pyinittypes(f2pysetupfunc)
      use types, only : dp
      external f2pysetupfunc
      call f2pysetupfunc(dp)
      end subroutine f2pyinittypes

      subroutine f2pywrap_fisher_exact_fexact (nrow, ncol, table, ldtabl&
     &, expect, percnt, emin, wkspace, prt, pre, f2py_table_d0, f2py_tab&
     &le_d1)
      use fisher_exact, only : fexact
      integer nrow
      integer ncol
      integer ldtabl
      real(kind=8) expect
      real(kind=8) percnt
      real(kind=8) emin
      integer wkspace
      real(kind=8) prt
      real(kind=8) pre
      integer f2py_table_d0
      integer f2py_table_d1
      real(kind=8) table(f2py_table_d0,f2py_table_d1)
      call fexact(nrow, ncol, table, ldtabl, expect, percnt, emin, wkspa&
     &ce, prt, pre)
      end subroutine f2pywrap_fisher_exact_fexact
      subroutine f2pywrap_fisher_exact_f2xact (nrow, ncol, table, ldtabl&
     &, expect, percnt, emin, prt, pre, ldkey, ldstp, ntotal, ncols, nro&
     &ws, f2py_table_d0, f2py_table_d1)
      use fisher_exact, only : f2xact
      integer nrow
      integer ncol
      integer ldtabl
      real(kind=8) expect
      real(kind=8) percnt
      real(kind=8) emin
      real(kind=8) prt
      real(kind=8) pre
      integer ldkey
      integer ldstp
      integer ntotal
      integer ncols
      integer nrows
      integer f2py_table_d0
      integer f2py_table_d1
      real(kind=8) table(f2py_table_d0,f2py_table_d1)
      call f2xact(nrow, ncol, table, ldtabl, expect, percnt, emin, prt, &
     &pre, ldkey, ldstp, ntotal, ncols, nrows)
      end subroutine f2pywrap_fisher_exact_f2xact
      subroutine f2pywrap_fisher_exact_f3xact (nrow, irow, ncol, icol, d&
     &lp, mm, fact, tol, f2py_irow_d0, f2py_icol_d0)
      use fisher_exact, only : f3xact
      integer nrow
      integer ncol
      real(kind=8) dlp
      integer mm
      real(kind=8) tol
      integer f2py_irow_d0
      integer f2py_icol_d0
      integer irow(f2py_irow_d0)
      integer icol(f2py_icol_d0)
      real(kind=8) fact(1)
      call f3xact(nrow, irow, ncol, icol, dlp, mm, fact, tol)
      end subroutine f2pywrap_fisher_exact_f3xact
      subroutine f2pywrap_fisher_exact_f4xact (nrow, irow, ncol, icol, d&
     &sp, fact, tol, f2py_irow_d0, f2py_icol_d0)
      use fisher_exact, only : f4xact
      integer nrow
      integer ncol
      real(kind=8) dsp
      real(kind=8) tol
      integer f2py_irow_d0
      integer f2py_icol_d0
      integer irow(f2py_irow_d0)
      integer icol(f2py_icol_d0)
      real(kind=8) fact(1)
      call f4xact(nrow, irow, ncol, icol, dsp, fact, tol)
      end subroutine f2pywrap_fisher_exact_f4xact
      subroutine f2pywrap_fisher_exact_f5xact (pastp, tol, kval, key, ld&
     &key, ipoin, stp, ldstp, ifrq, npoin, nr, nl, ifreq, itop, ipsh, f2&
     &py_key_d0, f2py_ipoin_d0, f2py_stp_d0, f2py_ifrq_d0, f2py_npoin_d0&
     &, f2py_nr_d0, f2py_nl_d0)
      use fisher_exact, only : f5xact
      real(kind=8) pastp
      real(kind=8) tol
      integer kval
      integer ldkey
      integer ldstp
      integer ifreq
      integer itop
      logical ipsh
      integer f2py_key_d0
      integer f2py_ipoin_d0
      integer f2py_stp_d0
      integer f2py_ifrq_d0
      integer f2py_npoin_d0
      integer f2py_nr_d0
      integer f2py_nl_d0
      integer key(f2py_key_d0)
      integer ipoin(f2py_ipoin_d0)
      real(kind=8) stp(f2py_stp_d0)
      integer ifrq(f2py_ifrq_d0)
      integer npoin(f2py_npoin_d0)
      integer nr(f2py_nr_d0)
      integer nl(f2py_nl_d0)
      call f5xact(pastp, tol, kval, key, ldkey, ipoin, stp, ldstp, ifrq,&
     & npoin, nr, nl, ifreq, itop, ipsh)
      end subroutine f2pywrap_fisher_exact_f5xact
      subroutine f2pywrap_fisher_exact_f6xact (nrow, irow, iflag, kyy, k&
     &ey, ldkey, last, ipn, f2py_irow_d0, f2py_kyy_d0, f2py_key_d0)
      use fisher_exact, only : f6xact
      integer nrow
      integer iflag
      integer ldkey
      integer last
      integer ipn
      integer f2py_irow_d0
      integer f2py_kyy_d0
      integer f2py_key_d0
      integer irow(f2py_irow_d0)
      integer kyy(f2py_kyy_d0)
      integer key(f2py_key_d0)
      call f6xact(nrow, irow, iflag, kyy, key, ldkey, last, ipn)
      end subroutine f2pywrap_fisher_exact_f6xact
      subroutine f2pywrap_fisher_exact_f7xact (nrow, imax, idif, k, ks, &
     &iflag, f2py_imax_d0, f2py_idif_d0)
      use fisher_exact, only : f7xact
      integer nrow
      integer k
      integer ks
      integer iflag
      integer f2py_imax_d0
      integer f2py_idif_d0
      integer imax(f2py_imax_d0)
      integer idif(f2py_idif_d0)
      call f7xact(nrow, imax, idif, k, ks, iflag)
      end subroutine f2pywrap_fisher_exact_f7xact
      subroutine f2pywrap_fisher_exact_f8xact (irow, is, i1, izero, new_&
     &bn, f2py_irow_d0, f2py_new_bn_d0)
      use fisher_exact, only : f8xact
      integer is
      integer i1
      integer izero
      integer f2py_irow_d0
      integer f2py_new_bn_d0
      integer irow(f2py_irow_d0)
      integer new_bn(f2py_new_bn_d0)
      call f8xact(irow, is, i1, izero, new_bn)
      end subroutine f2pywrap_fisher_exact_f8xact
      subroutine f2pywrap_fisher_exact_f9xact (f9xactf2pywrap, n, mm, ir&
     &, fact, f2py_ir_d0)
      use fisher_exact, only : f9xact
      integer n
      integer mm
      integer f2py_ir_d0
      integer ir(f2py_ir_d0)
      real(kind=8) fact(1)
      real(kind=8) f9xactf2pywrap
      f9xactf2pywrap = f9xact(n, mm, ir, fact)
      end subroutine f2pywrap_fisher_exact_f9xact
      subroutine f2pywrap_fisher_exact_f10act (nrow, irow, ncol, icol, v&
     &al, xmin, fact, nd, ne, m, f2py_irow_d0, f2py_icol_d0, f2py_nd_d0,&
     & f2py_ne_d0, f2py_m_d0)
      use fisher_exact, only : f10act
      integer nrow
      integer ncol
      real(kind=8) val
      logical xmin
      integer f2py_irow_d0
      integer f2py_icol_d0
      integer f2py_nd_d0
      integer f2py_ne_d0
      integer f2py_m_d0
      integer irow(f2py_irow_d0)
      integer icol(f2py_icol_d0)
      real(kind=8) fact(1)
      integer nd(f2py_nd_d0)
      integer ne(f2py_ne_d0)
      integer m(f2py_m_d0)
      call f10act(nrow, irow, ncol, icol, val, xmin, fact, nd, ne, m)
      end subroutine f2pywrap_fisher_exact_f10act
      subroutine f2pywrap_fisher_exact_f11act (irow, i1, i2, new_bn, f2p&
     &y_irow_d0, f2py_new_bn_d0)
      use fisher_exact, only : f11act
      integer i1
      integer i2
      integer f2py_irow_d0
      integer f2py_new_bn_d0
      integer irow(f2py_irow_d0)
      integer new_bn(f2py_new_bn_d0)
      call f11act(irow, i1, i2, new_bn)
      end subroutine f2pywrap_fisher_exact_f11act
      subroutine f2pywrap_fisher_exact_isort (n, ix, f2py_ix_d0)
      use fisher_exact, only : isort
      integer n
      integer f2py_ix_d0
      integer ix(f2py_ix_d0)
      call isort(n, ix)
      end subroutine f2pywrap_fisher_exact_isort
      subroutine f2pywrap_fisher_exact_gammad (gammadf2pywrap, x, p)
      use fisher_exact, only : gammad
      real(kind=8) x
      real(kind=8) p
      real(kind=8) gammadf2pywrap
      gammadf2pywrap = gammad(x, p)
      end subroutine f2pywrap_fisher_exact_gammad
      subroutine f2pywrap_fisher_exact_alnorm (alnormf2pywrap, x, upper)
      use fisher_exact, only : alnorm
      real(kind=8) x
      logical upper
      real(kind=8) alnormf2pywrap
      alnormf2pywrap = alnorm(x, upper)
      end subroutine f2pywrap_fisher_exact_alnorm
      subroutine f2pywrap_fisher_exact_lngamma (lngammaf2pywrap, z)
      use fisher_exact, only : lngamma
      real(kind=8) z
      real(kind=8) lngammaf2pywrap
      lngammaf2pywrap = lngamma(z)
      end subroutine f2pywrap_fisher_exact_lngamma
      
      subroutine f2pyinitfisher_exact(f2pysetupfunc)
      interface 
      subroutine f2pywrap_fisher_exact_fexact (nrow, ncol, table, ldtabl&
     &, expect, percnt, emin, wkspace, prt, pre, f2py_table_d0, f2py_tab&
     &le_d1)
      integer nrow
      integer ncol
      integer ldtabl
      real(kind=8) expect
      real(kind=8) percnt
      real(kind=8) emin
      integer wkspace
      real(kind=8) prt
      real(kind=8) pre
      integer f2py_table_d0
      integer f2py_table_d1
      real(kind=8) table(f2py_table_d0,f2py_table_d1)
      end subroutine f2pywrap_fisher_exact_fexact 
      subroutine f2pywrap_fisher_exact_f2xact (nrow, ncol, table, ldtabl&
     &, expect, percnt, emin, prt, pre, ldkey, ldstp, ntotal, ncols, nro&
     &ws, f2py_table_d0, f2py_table_d1)
      integer nrow
      integer ncol
      integer ldtabl
      real(kind=8) expect
      real(kind=8) percnt
      real(kind=8) emin
      real(kind=8) prt
      real(kind=8) pre
      integer ldkey
      integer ldstp
      integer ntotal
      integer ncols
      integer nrows
      integer f2py_table_d0
      integer f2py_table_d1
      real(kind=8) table(f2py_table_d0,f2py_table_d1)
      end subroutine f2pywrap_fisher_exact_f2xact 
      subroutine f2pywrap_fisher_exact_f3xact (nrow, irow, ncol, icol, d&
     &lp, mm, fact, tol, f2py_irow_d0, f2py_icol_d0)
      integer nrow
      integer ncol
      real(kind=8) dlp
      integer mm
      real(kind=8) tol
      integer f2py_irow_d0
      integer f2py_icol_d0
      integer irow(f2py_irow_d0)
      integer icol(f2py_icol_d0)
      real(kind=8) fact(1)
      end subroutine f2pywrap_fisher_exact_f3xact 
      subroutine f2pywrap_fisher_exact_f4xact (nrow, irow, ncol, icol, d&
     &sp, fact, tol, f2py_irow_d0, f2py_icol_d0)
      integer nrow
      integer ncol
      real(kind=8) dsp
      real(kind=8) tol
      integer f2py_irow_d0
      integer f2py_icol_d0
      integer irow(f2py_irow_d0)
      integer icol(f2py_icol_d0)
      real(kind=8) fact(1)
      end subroutine f2pywrap_fisher_exact_f4xact 
      subroutine f2pywrap_fisher_exact_f5xact (pastp, tol, kval, key, ld&
     &key, ipoin, stp, ldstp, ifrq, npoin, nr, nl, ifreq, itop, ipsh, f2&
     &py_key_d0, f2py_ipoin_d0, f2py_stp_d0, f2py_ifrq_d0, f2py_npoin_d0&
     &, f2py_nr_d0, f2py_nl_d0)
      real(kind=8) pastp
      real(kind=8) tol
      integer kval
      integer ldkey
      integer ldstp
      integer ifreq
      integer itop
      logical ipsh
      integer f2py_key_d0
      integer f2py_ipoin_d0
      integer f2py_stp_d0
      integer f2py_ifrq_d0
      integer f2py_npoin_d0
      integer f2py_nr_d0
      integer f2py_nl_d0
      integer key(f2py_key_d0)
      integer ipoin(f2py_ipoin_d0)
      real(kind=8) stp(f2py_stp_d0)
      integer ifrq(f2py_ifrq_d0)
      integer npoin(f2py_npoin_d0)
      integer nr(f2py_nr_d0)
      integer nl(f2py_nl_d0)
      end subroutine f2pywrap_fisher_exact_f5xact 
      subroutine f2pywrap_fisher_exact_f6xact (nrow, irow, iflag, kyy, k&
     &ey, ldkey, last, ipn, f2py_irow_d0, f2py_kyy_d0, f2py_key_d0)
      integer nrow
      integer iflag
      integer ldkey
      integer last
      integer ipn
      integer f2py_irow_d0
      integer f2py_kyy_d0
      integer f2py_key_d0
      integer irow(f2py_irow_d0)
      integer kyy(f2py_kyy_d0)
      integer key(f2py_key_d0)
      end subroutine f2pywrap_fisher_exact_f6xact 
      subroutine f2pywrap_fisher_exact_f7xact (nrow, imax, idif, k, ks, &
     &iflag, f2py_imax_d0, f2py_idif_d0)
      integer nrow
      integer k
      integer ks
      integer iflag
      integer f2py_imax_d0
      integer f2py_idif_d0
      integer imax(f2py_imax_d0)
      integer idif(f2py_idif_d0)
      end subroutine f2pywrap_fisher_exact_f7xact 
      subroutine f2pywrap_fisher_exact_f8xact (irow, is, i1, izero, new_&
     &bn, f2py_irow_d0, f2py_new_bn_d0)
      integer is
      integer i1
      integer izero
      integer f2py_irow_d0
      integer f2py_new_bn_d0
      integer irow(f2py_irow_d0)
      integer new_bn(f2py_new_bn_d0)
      end subroutine f2pywrap_fisher_exact_f8xact 
      subroutine f2pywrap_fisher_exact_f9xact (f9xactf2pywrap, f9xact, n&
     &, mm, ir, fact, f2py_ir_d0)
      real(kind=8) f9xact
      integer n
      integer mm
      integer f2py_ir_d0
      integer ir(f2py_ir_d0)
      real(kind=8) fact(1)
      real(kind=8) f9xactf2pywrap
      end subroutine f2pywrap_fisher_exact_f9xact 
      subroutine f2pywrap_fisher_exact_f10act (nrow, irow, ncol, icol, v&
     &al, xmin, fact, nd, ne, m, f2py_irow_d0, f2py_icol_d0, f2py_nd_d0,&
     & f2py_ne_d0, f2py_m_d0)
      integer nrow
      integer ncol
      real(kind=8) val
      logical xmin
      integer f2py_irow_d0
      integer f2py_icol_d0
      integer f2py_nd_d0
      integer f2py_ne_d0
      integer f2py_m_d0
      integer irow(f2py_irow_d0)
      integer icol(f2py_icol_d0)
      real(kind=8) fact(1)
      integer nd(f2py_nd_d0)
      integer ne(f2py_ne_d0)
      integer m(f2py_m_d0)
      end subroutine f2pywrap_fisher_exact_f10act 
      subroutine f2pywrap_fisher_exact_f11act (irow, i1, i2, new_bn, f2p&
     &y_irow_d0, f2py_new_bn_d0)
      integer i1
      integer i2
      integer f2py_irow_d0
      integer f2py_new_bn_d0
      integer irow(f2py_irow_d0)
      integer new_bn(f2py_new_bn_d0)
      end subroutine f2pywrap_fisher_exact_f11act 
      subroutine f2pywrap_fisher_exact_isort (n, ix, f2py_ix_d0)
      integer n
      integer f2py_ix_d0
      integer ix(f2py_ix_d0)
      end subroutine f2pywrap_fisher_exact_isort 
      subroutine f2pywrap_fisher_exact_gammad (gammadf2pywrap, gammad, x&
     &, p)
      real(kind=8) gammad
      real(kind=8) x
      real(kind=8) p
      real(kind=8) gammadf2pywrap
      end subroutine f2pywrap_fisher_exact_gammad 
      subroutine f2pywrap_fisher_exact_alnorm (alnormf2pywrap, alnorm, x&
     &, upper)
      real(kind=8) alnorm
      real(kind=8) x
      logical upper
      real(kind=8) alnormf2pywrap
      end subroutine f2pywrap_fisher_exact_alnorm 
      subroutine f2pywrap_fisher_exact_lngamma (lngammaf2pywrap, lngamma&
     &, z)
      real(kind=8) lngamma
      real(kind=8) z
      real(kind=8) lngammaf2pywrap
      end subroutine f2pywrap_fisher_exact_lngamma
      end interface
      external f2pysetupfunc
      call f2pysetupfunc(f2pywrap_fisher_exact_fexact,f2pywrap_fisher_ex&
     &act_f2xact,f2pywrap_fisher_exact_f3xact,f2pywrap_fisher_exact_f4xa&
     &ct,f2pywrap_fisher_exact_f5xact,f2pywrap_fisher_exact_f6xact,f2pyw&
     &rap_fisher_exact_f7xact,f2pywrap_fisher_exact_f8xact,f2pywrap_fish&
     &er_exact_f9xact,f2pywrap_fisher_exact_f10act,f2pywrap_fisher_exact&
     &_f11act,f2pywrap_fisher_exact_isort,f2pywrap_fisher_exact_gammad,f&
     &2pywrap_fisher_exact_alnorm,f2pywrap_fisher_exact_lngamma)
      end subroutine f2pyinitfisher_exact


